@@ -52,7 +52,15 @@ if not feedparser._XML_AVAILABLE:
     sys.stderr.write('No XML parsers available, unit testing can not proceed\n')
     sys.exit(1)
 
-_UTF32_AVAILABLE = feedparser._UTF32_AVAILABLE
+try:
+    # the utf_32 codec was introduced in Python 2.6; it's necessary to
+    # check this as long as feedparser supports Python 2.4 and 2.5
+    codecs.lookup('utf_32')
+except LookupError:
+    _UTF32_AVAILABLE = False
+else:
+    _UTF32_AVAILABLE = True
+
 _s2bytes = feedparser._s2bytes
 _l2bytes = feedparser._l2bytes
 
@@ -266,6 +274,14 @@ class TestEncodings(BaseTestCase):
         doc = codecs.BOM_UTF16_BE + doc.encode('utf-16be')
         result = feedparser.parse(doc)
         self.assertEqual(result['feed']['title'], u'&amp;exponential3')
+    def test_gb2312_converted_to_gb18030_in_xml_encoding(self):
+        # \u55de was chosen because it exists in gb18030 but not gb2312
+        feed = u'''<?xml version="1.0" encoding="gb2312"?>
+                  <feed><title>\u55de</title></feed>'''
+        result = feedparser.parse(feed.encode('gb18030'), response_headers={
+            'Content-Type': 'text/xml'
+        })
+        self.assertEqual(result.encoding, 'gb18030')
 
 class TestFeedParserDict(unittest.TestCase):
     "Ensure that FeedParserDict returns values as expected and won't crash"
@@ -580,6 +596,9 @@ date_tests = {
         (u'Mon, 26 January 2004 16:31:00 MT', (2004, 1, 26, 23, 31, 0, 0, 26, 0)),
         (u'Mon, 26 January 2004 16:31:00 PT', (2004, 1, 27, 0, 31, 0, 1, 27, 0)),
     ),
+    feedparser._parse_date_rfc822_grubby: (
+        (u'Thu Aug 30 2012 17:26:16 +0200', (2012, 8, 30, 15, 26, 16, 3, 243, 0)),
+    ),
     feedparser._parse_date_asctime: (
         (u'Sun Jan  4 16:29:06 2004', (2004, 1, 4, 16, 29, 6, 6, 4, 0)),
     ),
@@ -655,6 +674,19 @@ class TestBuildRequest(unittest.TestCase):
           request.get_header('Cache-control'), 'max-age=0')
 
 
+class TestLxmlBug(unittest.TestCase):
+    def test_lxml_etree_bug(self):
+        try:
+            import lxml.etree
+        except ImportError:
+            pass
+        else:
+            doc = u"<feed>&illformed_charref</feed>".encode('utf8')
+            # Importing lxml.etree currently causes libxml2 to
+            # throw SAXException instead of SAXParseException.
+            feedparser.parse(feedparser._StringIO(doc))
+        self.assertTrue(True)
+
 #---------- parse test files and create test methods ----------
 def convert_to_utf8(data):
     "Identify data's encoding using its byte order mark" \
@@ -805,6 +837,7 @@ def runtests():
         testsuite.addTest(testloader.loadTestsFromTestCase(TestMakeSafeAbsoluteURI))
         testsuite.addTest(testloader.loadTestsFromTestCase(TestEverythingIsUnicode))
         testsuite.addTest(testloader.loadTestsFromTestCase(TestTemporaryFallbackBehavior))
+        testsuite.addTest(testloader.loadTestsFromTestCase(TestLxmlBug))
         testresults = unittest.TextTestRunner(verbosity=1).run(testsuite)
 
         # Return 0 if successful, 1 if there was a failure
